from . import product
from . import search_result 
from . import user
from . import cart 
from . import color
from . import comment
from . import question
from . import seller
from . import object_model



from .models.search_result import SearchResult
from .models.product import FullProduct
from .models.user import FullUser
from .models.cart import FullCart
from .models.comment import FullComments
from .search_filter import SearchFilter
from .import errors

import aiohttp , asyncio
import os , pickle 


class Client:
    _API_V1 = "https://api.digikala.com/v1/"
    _API_V2 = "https://api.digikala.com/v2/"
    
    def __init__(self,
                 name:str,
                 phone_number:str=None,
                 email:str = None,
                 password:str=None,
                 do_login:bool = None,
                 in_memory:bool=None,
                 workdir:str=None
                 
                 ) -> None:
        
        if phone_number:
            phone_number = phone_number.replace('+98','0')
        
        self.name = name
        self.session_path = workdir + name +'.digikala' if workdir else name +'.digikala'
        self.phone_number = phone_number
        self.email = email
        self.password = password
        self.do_login  = do_login
        self.in_memory = in_memory
        self._session = aiohttp.ClientSession()
        self._is_login = None
        
        self.cart_id : int = None
        
        
    async def logout(self)->bool:
        """
        Sign out of the user account

        Raises:
            errors.SessionExpired: if the session has already expired

        Returns:
            True: if done successfully
        """
        res = await self._session.post(self._API_V1+'user/logout/',json={})    
        if res.json()['status'] == 200: return True     
        raise errors.SessionExpired()


    async def login(self):
        """
        Start logging into the user account in the terminal
        """
        if self._is_login:
            print('Client is already running!')            
        
        else:
            
            if os.path.exists(self.session_path):
                await self.load_session()
                return
            
        
            while not self.phone_number and not self.email:
                username = input('Please enter your phone number or email to log in: ')
                username = username.replace('+98','0')
                
                if not username.strip(" "):
                    continue
                
                if username.isnumeric():
                    self.phone_number = username
                    break
                
                if username.find('@'):
                    self.email = username
                    break
                
                print('The wrong phone number or email was entered!!')
            
            
            if not await self.send_code():
                raise errors.UsernameInvalid()
            
            while True:
                
                if self.password:
                    try:
                        await self.signin_with_password()
                    except errors.SessionWrongPassword as e:
                        self.password = None
                        print(e)
                        continue
                    
                else:    
                    code = input('Please enter the code sent to your phone number or email(enter (r) to resubmit the code!): ')
                    if code == 'r':
                        if not await self.resend_code(): 
                            print('The code could not be sent, please try again in at least one minute!')
                        continue
                    
                    try:
                        await self.sign_in(code)
                            
                    except errors.SessionWrongCode:
                        print('The entered code is wrong!')
                        
                print('Login is done.')
                break

            
    async def resend_code(self)->bool:
        """
        Get the login code again.
        
        Returns:
            True: if done successfully
        
        """
        res = await self._session.post(self._API_V1+'user/authenticate/',json={"backUrl":"/","username":self.phone_number or self.email,"otp_call":False,"force_send_otp":True})
        res = await res.json()
        return True if res['status'] == 200 else False


    async def signin_with_password(self) -> FullUser:
        """
        Go to user account with password

        Raises:
            errors.SessionExpired: if sign in the account is not logged in correctly or the session is blocked
            errors.SessionWrongPassword: if the entered password is wrong

        Returns:
            models.user.FullUser
        """
        res = await self._session.post(self._API_V1+'user/login/password/',json={"backUrl":"/profile/personal-info/","type":"password","username":self.phone_number or self.email,"password":self.password})
        res = await res.json()
        if res['status'] == 200 :
            me = await self.me()
            
            if me and me.is_logged_in:
                await self.save_session()
                self.cart_id = me.cart.id
                return me
            
            raise errors.SessionExpired()
        
        raise errors.SessionWrongPassword()


    async def check_signin(self)-> bool:
        """
        Check if you are logged in or not!

        Returns:
            True: If you are logged in
        """
        me = await self.me()
        return me.is_logged_in
    
    
    async def send_code(self) -> bool:
        """
        Send code to phone number or email

        Returns:
            True: if the code is sent
        """
        res = await self._session.post(self._API_V1+'user/authenticate/',json={"backUrl":"/","username":self.phone_number or self.email,"otp_call":False})
        res = await res.json()
        return True if res['status'] == 200 else False


    async def sign_in(self,code:str) -> FullUser:
        """
        Login to the user account with the SMS code sent to the mobile number or email

        Args:
            code (`str`): Code sent to phone number or email

        Returns:
            models.user.FullUser
        
        Raises:
            errors.SessionExpired: if sign in the account is not logged in correctly or the session is blocked
            errors.SessionWrongCode: if the entered code is wrong
        
        """
        res = await self._session.post(self._API_V1+'user/login/otp/',json={"backUrl":"/","type":"otp","username":self.phone_number or self.email,"code":code})
        res = await res.json()
        if res['status'] == 200 :
            me = await self.me()
            
            if me and me.is_logged_in:
                await self.save_session()
                self.cart_id = me.cart.id
                return me
            
            raise errors.SessionExpired()
        
        raise errors.SessionWrongCode()
        

    async def save_session(self):
        """
        Create a session file
        """
        if not self.in_memory:
            with open(self.session_path,'wb') as session_file:
                session_data = {'cookies': self._session.cookie_jar._cookies,'phone_number':self.phone_number}
                pickle.dump(session_data,session_file)

    async def load_session(self):
        """
        Load a session file

        Raises:
            errors.SessionInvalid: if the session file is invalid
            errors.SessionExpired: if the session is expired
        """
        try:
            with open(self.session_path,'rb') as session_file:
                session_data = pickle.load(session_file)
            
            self._session.cookie_jar._cookies = session_data['cookies']
            self.phone_number = session_data['phone_number']
        except:
            raise errors.SessionInvalid()
        
        me = await self.me()
        if not me.is_logged_in:
            raise errors.SessionExpired()
        
        self.cart_id = me.cart.id
        

    async def me(self) -> FullUser:
        """
        Get your information

        Returns:
            models.user.FullUser
        """
        res = await self._session.get(self._API_V1+'user/init/?backUrl=%2F')
        res = await res.json()
        return FullUser(res['data'],'fulluser',self) if res['status'] == 200 else None

        
    
    async def search(self,filter:SearchFilter) -> SearchResult:
        """
        Start your search to find products

        Args:
            filter: filter type for product search 

        Returns:
            models.search_result.SearchResult
        """
        
        
        res = await self._session.get(self._API_V1+filter.url,params=filter.params)
        res = await res.json()
        return SearchResult(res['data'],self,filter) if res['status'] == 200 else None
    
    
    async def get_product(self,product_id:int) -> FullProduct:
        """
        Search for a product using its ID

        Args:
            product_id (`int`): The unique ID of the product

        Returns:
            models.product.FullProduct
        """
        res = await self._session.get(self._API_V2+f'product/{product_id}/')
        res = await res.json()
        return FullProduct(res['data'],'fullproduct',self) if res['status'] == 200 else None
    
    async def add_cart(self,variant_id:int) -> FullCart:
        """
        Add an item to the cart
        
        Args:
            variant_id (`int`): The unique identifier of the product (note: this identifier is different from the product_id)

        Returns:
            models.cart.FullCart
        """
        data = {"variant_ids":[variant_id],"pass":True}
        res = await self._session.post(self._API_V1+'cart/add/',json=data)
        res = await res.json()
        return FullCart(res['data'],'fullcart',self) if res['status'] == 200 else None
        
    async def get_cart(self) -> FullCart:
        """
        Get shopping cart information
        
        Returns:
            models.cart.FullCart
        """
        res = await self._session.get(self._API_V1+'cart/')
        res = await res.json()
        return FullCart(res['data'],'fullcart',self) if res['status'] == 200 else None
        
    
    async def remove_item_cart(self,cart_item_id:int) -> FullCart:
        """
        Remove an item from the shopping cart

        Args:
            cart_item_id (`int`): ID of the item in the shopping cart

        Returns:
            models.cart.FullCart
        """
        data = {"web_page":"cart","cart_id":self.cart_id,"cart_item_ids":[cart_item_id]}
        res = await self._session.post(self._API_V1+"cart/remove/",json=data)
        res = await res.json()
        return FullCart(res['data'],'fullcart',self) if res['status'] == 200 else None

    
    async def get_comments(self,product_id:int,page:int=1) -> FullComments:
        """
        Get comments on a product
        
        Args:
            product_id (`int`): ID of the desired item 
            page (`int`, optional): The desired page of product comments. (Defaults to 1).

        Returns:
            models.comment.FullComments
        """
        
        res = await self._session.get(self._API_V1+f"rate-review/products/{product_id}/?page={page}")
        res = await res.json()
        return FullComments(res['data'],'fullcomments',self) if res['status'] == 200 else None
    
    
    # ::::::::::::::::::::
    async def __aenter__(self):
        if self.do_login:
            await self.login()
        return self

    async def __aexit__(self, *args, **kwargs):
        await self._session.close()

    async def close(self):
        """stop client"""
        await self.__aexit__()



from .object_model import Object    
from ._cart import *

class Cart(Object):
    id : int
    items_count : int
    payable_price : int
    digiclub : CartDigiClub
    voucher_code : int
    cash_back : CashBack
    temporary_plus_subscription : list
    bnpl_active : bool
    rrp_price : int
    shipping_cost : int
    voucher_discount : int
    shipping_cost_discount : int
    items_discount : int
    total_discount : int
    insurance : Insurance
    plus : list
    cart_items : list[CartItem]



class FullCart(Object):
    cart_items : list[CartItem] 
    cart : Cart
    digiplus : Digiplus
    data_layer : list[DataLayer]
    recommendation : Recommendation
    new_recommendation : Recommendation
    bigdata_tracker_data : BigdataTrackerData
    intrack : Intrack




from typing import Any
from .object_model import Object
from .cart import Cart


class DigiClub(Object):
    is_digiclub_activated : bool
    points : int
    reward_url_threshold : int
    claimable_points : int

class DigiPlus(Object):
    subscription_remaining_days : int
    is_activated : bool
    user_state : str
    is_general_location_jet_eligible : bool

class notification_(Object):
    count : int

class Notification(Object):
    notification: notification_


# FullUser


class DataLayerData(Object):
    is_logged_in : bool
    user_id : int
    is_club_user : bool
    club_points :int
    rfm_category : Any
    total_delivered_orders : int
    aov :int
    magnet_membership : bool
    plus_membership : bool 
    plus_membership_duration : int
    
class DataLayer(Object):
    event : str 
    data : DataLayerData

class SocialProfile(Object):
    is_activated : bool



from .object_model import Object
from ._comment import *


class Comment(Object):
    order_item_id : int
    product_id : int
    rate : int
    review_user_type : str
    status : str
    purchased_item : PurchasedItem
    id : int
    body : str
    created_at : str 
    rate : int
    reactions : CommandReaction
    recommendation_status : str 
    is_buyer : bool 
    user_name : str 
    is_anonymous : bool 
    relative_date : str
    files : list[Files]


class IntentData(Object):
    title : str
    number_of_comments : int
    tag_data : Tag
    tag_percentage : Tag
    productId : int
    intentId : int
    
    
class FullComments(Object):
    comments : list[Comment]
    intent_data : list[IntentData]
    media_comments : list[Comment]
    sort : Sort
    sort_options : list[SortOptions]
    pager : Pager
    
    async def get_next_page(self):
        "دریافت کامنت های صفحه بعدی | Getting next comments"
        await self._client.get_comments(self.intent_data[0].productId,self.pager.current_page+1)
    


from .object_model import Object
from ._user import *

class User(Object):
    id : int
    first_name : str
    last_name : str 
    phone : str
    city_id : int
    mobile : str
    email : str
    avatar_url : str
    is_legal : bool
    is_foreigner : bool
    has_password : bool
    personal_economic_number : int 
    verification_status : str
    national_identity_number : int
    gender : str
    birthday_iso : str
    city_name : str
    state_name : str
    phone_hash : str
       

class FullUser(Object):
    is_logged_in : bool
    digiclub : DigiClub
    digiplus : DigiPlus
    notification : Notification
    user : User
    default_address : list
    data_layer : DataLayer
    show_intrack_web_push : bool
    modules : list[str]
    date_time : str
    social_profile : SocialProfile
    cart : Cart
    
    
    
    


from typing import Any
from .object_model import Object


class SellerRating(Object): 
    total_rate : int  
    total_count : int  
    commitment : int  
    no_return : float  
    on_time_shipping : int  

class SellerProperties(Object):
   is_trusted : bool  
   is_official : bool  
   is_roosta : bool  
   is_new : bool   

class Grade(Object):
    label : str  
    color : str  


class Seller(Object):
    id : int  
    title : str  
    code : str  
    url : str  
    rate : float
    rating : SellerRating
    properties : SellerProperties  
    stars : int  
    grade : Grade
    logo : Any 
    registration_date : str  



from .product import Product

class SearchResult:
    def __init__(self,data:dict,client,filter) -> None :
        self.products : list[Product] =  [Product(i,'product',client) for i in data['products']]
        self.total_items : int = data['pager']['total_items']
        self.total_pages : int = data['pager']['total_pages']
        self.current_page : int = data['pager']['current_page']
        self.query : str = filter.params['q']
        self.filter = filter
        self._client = client

    async def next_page(self):
        """برای دریافت صفحه بعدی این جستجو از این متود استفاده کنید | Use this method to get the next page of this search"""
        _filter = self.filter
        _filter.params['page'] += 1
        return await self._client.search(_filter)
        
    
    async def get_page(self,page:int|str):
        """برای دریافت یک صفحه خاص در این جستجو از این متود استفاده کنید و شماره صفحه را به عنوان ورودی ارسال کنید | To get a specific page in this search, use this method and pass the page number as input"""
        
        _filter = self.filter
        _filter.params['page'] = page
        return await self._client.search(_filter)


    def __str__(self) -> str:
        return f"""
query : {self.query}
current_page : {self.current_page}
total_pages : {self.total_pages}
total_items : {self.total_items}
products : {self.products}
"""



from .object_model import Object
from .color import Color
from .seller import Seller

class CommandReaction(Object):
    likes : int 
    dislikes : int 

class PurchasedItem(Object):
    seller : Seller
    color : Color

class Tag(Object):
    positive : int 
    negative : int
    neutral : int

class Files(Object):
    storage_ids : list[str]
    url : list[str]
    thumbnail_url : list[str]
    
class Sort(Object):
    default : str

class SortOptions(Object):
    id : str
    title : str
    
class Pager(Object):
    current_page : int
    total_pages : int
    total_items : int






class SessionPasswordNeeded(Exception):
    def __init__(self) -> None:
        message = 'a session password is required' 
        super().__init__(message)


class SessionWrongCode(Exception):
    def __init__(self) -> None:
        message = 'The code is wrong' 
        super().__init__(message)

class SessionWrongPassword(Exception):
    def __init__(self) -> None:
        message = 'The Password is wrong' 
        super().__init__(message)



class UsernameInvalid(Exception):
    def __init__(self) -> None:
        message = 'The phone number or email is invalid' 
        super().__init__(message)





from .object_model import Object
from .product import Product , Variant , Digiplus


class CartDigiClub(Object):
    points : int
 
class CashBack(Object):
    amount : int 
    digiplus_amount : int 
    return_days : int
 
class Insurance(Object):
    amount : int 
    discount : int

class Price(Object):
    price : int
    discount : int 
    is_incredible : bool
    is_promotion : bool
    is_lightening_deal : bool
    is_digiplus_promotion : bool 
    
class Properties(Object):
    is_drop_off_eligible : bool

class ShipmentMethod(Object):
    is_digikala : bool
    is_ship_by_seller : bool    

class PriceDecreased(Object):
    to : int
    from_price : int
    def __init__(self, data: dict, name: str, client) -> None:
        self.from_price : int = data['from']
        super().__init__(data, name, client)
        
class Warnings(Object):
    price_decreased : PriceDecreased

class CartItem(Object):
    id : int
    cart_id : int
    quantity : int 
    price : Price
    product : Product
    variant : Variant
    properties : Properties
    shipment_methods : ShipmentMethod
    is_switchable : bool
    gifts : list
    has_insurance : bool
    e_gift_card_properties : list
    warnings : list[Warnings]



# FullCart 


class ActionField(Object):
    step : int
    option : str

class CheckoutOption(Object):
    actionField : ActionField

class CheckoutProduct(Object):
    brand : str
    category : list[str]
    metric6 : int
    dimension2 : int
    dimension6 : int
    dimension7 : str
    dimension9 : float
    dimension11 : int
    dimension20 : str
    item_category2 : str
    item_category3 : str
    item_category4 : str
    item_category5 : str
    name : str
    id : int
    price : int
    quantity : int
    list : str
    metric8 : int
    dimension3 : str
    dimension10 : int
    dimension15 : int
    metric15 : int

class Checkout(Object):
    actionField : ActionField
    products : list[CheckoutProduct]
    
class Ecommerce(Object):
    checkout : Checkout
    checkout_option : CheckoutOption

class DataLayer(Object):
    event : str
    ecommerce : Ecommerce

class Recommendation(Object):
    title : str
    products : list[Product]

class PageInfo(Object):
    cart_id : int

class BigdataTrackerData(Object):
    page_name : str
    page_info : PageInfo

class Intrack(Object):
    eventName : str
    userId : int



from typing import Any 
from .object_model import Object
from .color import Color
from .seller import Seller


class DataLayer(Object) :
    brand : str  
    category : list[str]  
    metric6 : int  
    dimension2 : int  
    dimension6 : int|float  
    dimension7 : str  
    dimension9 : int  
    dimension11 : int|float  
    dimension20 : str  
    item_category2 : str  
    item_category3 : str  
    item_category4 : str  
    item_category5 : str  

class Digiplus(Object):
    services : list[str]  
    is_jet_eligible : bool  
    cash_back : int|float  
    is_general_location_jet_eligible : bool  
    fast_shipping_text : str  

class ImagesMain(Object):
    storage_ids : list  
    url : list[str]  
    thumbnail_url : Any
    temporary_id : Any
    webp_url :list[str]  
  
class Images(Object): 
    main : ImagesMain  

class Rating(Object): 
    rate : float|int  
    count : int  

class StatisticsRating(Object) :
    rate_count : int  
    rate : float  

class Statistics(Object):
    totally_satisfied : StatisticsRating  
    satisfied : StatisticsRating  
    neutral : StatisticsRating  
    dissatisfied : StatisticsRating  
    totally_dissatisfied : StatisticsRating  
    total_count : int
    total_rate : float

class VariantProperties(Object):
    is_fast_shipping : bool  
    is_ship_by_seller : bool  
    is_multi_warehouse : bool  
    has_similar_variants : bool  
    is_rural : bool  
    in_digikala_warehouse : bool  
   
class Warranty(Object):
    id : int  
    title_fa : str  
    title_en : str  
   
   
class Digiclub(Object): 
    point : int  

class Badge(Object):
    title : str  
    color : str  
    icon : Any

class Price(Object):
    selling_price : int  
    rrp_price : int  
    order_limit : int  
    is_incredible : bool  
    is_promotion : bool  
    is_locked_for_digiplus : bool  
    bnpl_active : bool  
    marketable_stock : int  
    discount_percent : int  
    badge : Badge  
    is_digiplus_promotion : bool  
    is_digiplus_early_access : bool  
    is_application_incredible : bool  
    is_lightening_deal : bool  
    is_plus_early_access : bool  

class Provider(Object):
    title : str  
    description : str  
    has_lead_time : bool  
    type : str  

class ShipmentMethods(Object):
    description : str  
    has_lead_time : bool  
    providers : list[Provider] 

class Payload(Object): 
    text : str  
    text_color : str  
    icon : Any

class VariantBadge(Object):
    id : int  
    type : str  
    slot : str  
    priority : int   
    payload : Payload  

class Size(Object):
    id : int  
    title : str  


class Variant(Object): 
    id :int   
    lead_time : int|float  
    rank : float  
    rate : int  
    statistics : Statistics  
    status : str  
    properties : VariantProperties    
    digiplus : Digiplus   
    warranty : Warranty  
    color : Color  
    seller : Seller  
    digiclub : Digiclub  
    buy_box_lead_time : int|float  
    price : Price  
    shipment_methods : ShipmentMethods  
    has_importer_price : bool  
    manufacture_price_not_exist : bool  
    has_best_price_in_last_month : bool  
    buy_box_notices : list  
    variant_badges : list[VariantBadge]  
    size : Size  
    
class Properties(Object):
    is_fast_shipping : bool  
    is_ship_by_seller : bool  
    free_shipping_badge : bool  
    is_multi_warehouse : bool  
    is_fake : bool  
    has_gift : bool  
    min_price_in_last_month : int  
    is_non_inventory : bool  
    is_ad : bool  
    ad : list  
    is_jet_eligible : bool  
    is_medical_supplement : bool  

class Category(Object):
    id : int
    title_fa : str
    title_en : str
    code : str
    return_reason_alert : str

class Url(Object):
    uri : str


class Logo(Object):
    url : list[str]

class Brand(Object):
    id : int
    code : str
    title_fa : str
    title_en : str
    url : Url
    visibility : bool
    logo : Logo
    is_premium : bool
    is_miscellaneous : bool
    is_name_similar : bool

class Reviewattribute(Object):
    title : str
    values : list[str]

class Review(Object):
    description : str
    attributes : list[Reviewattribute]

class Breadcrumb(Object):
    title : str
    url : Url

class Specification(Object):
    title : str
    attributes : list[Reviewattribute]

class Section(Object):
    template : str
    image : str

class ReviewSection(Object):
    title : str
    sections : list[Section]

class ExpertReviews(Object):
    description : str
    short_review : str
    review_sections : list[ReviewSection]
    technical_properties : list

class BrandCategoryUrl(Object):
    uri : str

class Meta(Object):
    brand_category_url : BrandCategoryUrl

class ST_CmpTacker(Object):
    neo : str 
    cx : str 
    dx : str     
    data_fx : str 
    zero : str
    
    def __init__(self, data: dict, name: str, client) -> None:
        self.data_fx = self._data.get('data-fx',None)
        super().__init__(data, name, client)





# FullProduct :::::::::::::::::::::::::: 



class ProductDataLayer(Object) :
    brand : str  
    category : list[str]  
    metric6 : int  
    dimension2 : int  
    dimension6 : int|float  
    dimension7 : str  
    dimension9 : int  
    dimension11 : int|float  
    dimension20 : str  
    item_category2 : str  
    item_category3 : str  
    item_category4 : str  
    item_category5 : str  
    name : str  
    id : int  
    price : int   
    metric8 : int  
    dimension3 : str  
    dimension10	: int  
    dimension15	: int  
    metric15	: int  
    metric11	: int  
    metric12	: int  

class ActionField(Object) :
    list : str  

class Detail(Object):
    actionField : ActionField   	
    products : list[ProductDataLayer]  

class ECommerce(Object):
    detail : Detail  	
    

class FullData(Object):
    event : str  
    ecommerce : ECommerce  

class TwitterCard(Object):	
    title : str  
    image : str  
    price : int  
    description : str  

class OpenGraph(Object):	
    title : str  
    url : str  
    image : str  
    availability : str  
    type : str  
    site : str  
    price : int  
    description : Any

class Header(Object):
    title : str  
    description : str  
    canonical_url : str  

class MarkupSchemaBrand(Object):
    name : str  
    url	: str  
    
class Offers(Object):	
    priceCurrency : str  
    price : int  
    itemCondition : str  
    availability : str  
    
class ReviewRating(Object):	
        bestRating : int  
        ratingValue	: int     

class Author(Object):	
    name : str  

class Review(Object):	
    reviewRating : ReviewRating  	
    author : Author  	
    datePublished : str  
    reviewBody : str  
    name : str  


class MarkupSchema(Object):	
    name : str   
    alternateName : str  
    images : list[str]  
    description : str  
    mpn : int  
    sku : int  
    category : str  
    brand : MarkupSchemaBrand  
    aggregateRating	: Any 
    offers : Offers  
    review : Review  

class ItemListElement(Object):
    position : int  
    name : str  
    item : str  
    
class ItemListElementMarkupSchema(Object):
    itemListElement : list[ItemListElement]  
    
class Seo(Object):
    title : str  
    description	: str  
    twitter_card : TwitterCard  
    open_graph : OpenGraph  
    header : Header  
    markup_schema : list[MarkupSchema|list[ItemListElementMarkupSchema]]  


class EventData(Object):	
    currency : str  
    deviceType : str  
    name : str  
    productId : int  
    productImageUrl	: list[str]  
    quantity : int  
    leafCategory : str  
    unitPrice : int  
    url	: str  
    supplyCategory : str  
    categoryLevel1 : str  
    categoryLevel2 : str  
    categoryLevel3 : str  
    categoryLevel4 : str  
    categoryLevel5 : str  


class Intrack(Object):	
    eventName : str  
    eventData : EventData  	
    userId : Any

class PageInfo(Object):	
    product_id : int  
    
class BigdataTrackerData(Object):	
    page_name : str  
    page_info : PageInfo  




from typing import Any
from json import dumps
# from DigiKala import Client
import DigiKala

class Object:
    def __init__(self,data:dict,name:str,client:'DigiKala.Client') -> None:
        self._data = data
        self.__class__.__name__ = name
        self._client = client        
        self._dict_types = {
            'dict' : self._return_object,
            'str'  : self._return_str,
            'list' : self._return_list
        }

    
    
    def _return_object(self,value,name)-> "Object" :
        return Object(value,name,self._client)
    
    def _return_list(self,value,name)->list["Object" or "Any"]:
        return [Object(i,name,self._client) if isinstance(i,dict) else i for i in value]
    
    def _return_str(self,value,_)->str|list[str]:
        return [str(i) for i in value.strip('[]').split(',')] if value[0] == '[' else value 
    
    
    def __getattr__(self, key: str) -> Any:
        try:
    
            value = self._data[key]
            _type = value.__class__.__name__
            
            try:
                return self._dict_types[_type](value,key)
            except:
                return value
                
        except :
            return None
    
    
    @staticmethod
    def default(obj: "Object"):
        if isinstance(obj, bytes):
            return obj.decode()


        return {
            "->": obj.__class__.__name__,
            **{
                attr: obj._data[attr] for attr in  obj._data if obj._data[attr] is not None
            }
        }


    def __str__(self) -> str:
        return dumps(self, indent=4, default=Object.default, ensure_ascii=False)

    def __repr__(self) -> str:
        return dumps(self, indent=4, default=Object.default, ensure_ascii=False)
        




class SessionInvalid(Exception):
    def __init__(self) -> None:
        message = 'The session is invalid' 
        super().__init__(message)


class SessionExpired(Exception):
    def __init__(self) -> None:
        message = 'The session has expired'
        super().__init__(message)


from .comment import FullComments , Comment
from .question import Question
from ._product import *
import DigiKala    

class Product(Object):
    id : int  
    title_fa : str  
    title_en : str  
    url : Url
    status : str  
    has_quick_view : bool  
    data_layer : DataLayer  
    product_type : str  
    digiplus : Digiplus  
    images : Images  
    rating : Rating  
    default_variant : Variant  
    properties : Properties
    category : Category
    brand : Brand
    review : Review
    variants : list[Variant]
    badges : list  
    colors : list[Color] 
    questions_count : int
    comments_count : int
    breadcrumb : list[Breadcrumb]
    specifications : list[Specification]
    expert_reviews : ExpertReviews
    meta : Meta
    last_comments : list[Comment]
    last_questions : list[Question]
    digify_touchpoint : str 
    st_cmp_tacker : ST_CmpTacker
    
    async def get_product(self) -> 'FullProduct':
        "دریافت اطلاعات کامل محصول | Get complete product information"
        return await self._client.get_product(self.id)
    
    async def get_comments(self) -> 'FullComments':
        "دریافت کامنت ها | Get comments "
        return await self._client.get_comments(self.id)
    
    async def add_cart(self) -> 'DigiKala.models.cart.FullCart':
        "اضافه کردن محصول پیش فرض به سبد خرید | Add default product to cart"
        return await self._client.add_cart(self.default_variant.id)
    

class FullProduct(Object):
    product : Product  
    data_layer : FullData  
    seo : Seo  
    intrack : Intrack  
    landing_touchpoint : list  
    dynamic_touch_points : list  
    flat_shipping_badge	: str  
    promotion_banner : Any
    bigdata_tracker_data : BigdataTrackerData  




from setuptools import setup, find_packages

setup(
    name='DigiKala',
    version='1.0',
    packages=find_packages(),
    license='MIT',
    description='DigiKala library to connect and work with digikala.com!',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='NuoQTe',
    author_email='nuoqte@gmail.com',
    url='https://github.com/NuoQte/DigiKala',
    install_requires=['aiohttp']
)




class SearchFilter:
    params = {}
    url = 'search/'
    def __init__(self,
                query:str = None,
                classification : str = None,
                sort_code:int = None,
                page:int = 1,
                brands:list[int] =None,
                has_jet_delivery:bool = None,
                has_ship_by_seller:bool = None,
                has_selling_stock:bool = None,
                has_ready_to_shipment:bool = None,
                seller_types:list[str] = None,
                price_range: tuple[int,int]= None
                ) -> None:
        """
        Create a filter to search for content

        Args:
            query (`str`, optional): Text search
            classification (`str`, optional): Classification
            sort_code (`int`): Order by (The most relevant=22, most visited=4, the newest=1, Bestselling=7, cheapest=20, the most expensive=21, Fastest shipping=25, Buyers' suggestion=27, selected=29)
            page (`int`, optional): The desired page. (Defaults to 1).
            brands (`list`[`int`], optional): A list of brands
            has_jet_delivery (`bool`, optional): Shipping tomorrow
            has_ship_by_seller (`bool`, optional): Sent by seller
            has_selling_stock (`bool`, optional): Only available items
            has_ready_to_shipment (`bool`, optional): Only items available in Digikala warehouse
            seller_types (`str`, optional): A list of types of sellers. (official,trusted,digikala,roosta)
            price_range (`tuple`[`int`,`int`], optional): Price range(in Toman) --> (min , max)

        
        """
        
            
        if brands :
            for i in range(len(brands)) : self.params[f'brands[{i}]'] = brands[i] 

        if seller_types:     
            for i in range(len(seller_types)) :self.params[f'seller_types[{i}]'] = seller_types[i] 
   
        if classification : self.url + classification + "/"
        if query : self.params["q"] = query                 
        if has_jet_delivery : self.params["has_jet_delivery"] = 1
        if has_ship_by_seller : self.params["has_ship_by_seller"] = 1
        if has_selling_stock : self.params["has_selling_stock"] = 1
        if has_ready_to_shipment : self.params["has_ready_to_shipment"] = 1
        if price_range : self.params['price[min]'],self.params['price[max]'] = price_range[0],price_range[1]
        if sort_code : self.params["sort"] = sort_code 
        self.params['page'] = page

